#!/usr/bin/env python

import numpy as np
import rospy
from geometry_msgs.msg import Twist
from base_drive_18 import baseDrive


class actDrive(baseDrive):
    def __init__(self, cmd_vel_name):
        super().__init__(cmd_vel_name)
        self.scan_range = []

    def getScan3(self, scan_data3):
        super().getScan3(scan_data3)
        # 親クラスでself.getScan4()がコールバックされてself.scan_minが値を持つまで待つ
        while self.scan_data4 == []:
            rospy.sleep(0.01)
        # 以下のデータからなるscan_bothを生成する
        # scan_data3: 4, 19, 34, 49, 64, 79, 94, 109, 124, 139, 154, 169, 184
        # scan_data4: 19, 34, 49, 64, 79, 94, 109, 124, 139, 154, 169
        scan_both = scan_data3.ranges[4:185:15] \
            + self.scan_data4.ranges[19:170:15]
        # 無限大と非数を置換してself.scan_rangeを取得する
        scan_range = []
        for i in range(360 // 15):
            # infはで無限大(float型が取得る最大値を超える数)を表すfloat型オブジェクト
            if scan_both[i] == float('Inf'):
                scan_range.append(3.5)
            # nanは非数(欠損値)を表すfloat型オブジェクトでfloat('nan')で生成されるが、
            # 非数同士を==で比較してもFalseとなるため非数かどうかはnp.isnan()で判定
            elif np.isnan(scan_both[i]):
                scan_range.append(0)
            else:
                scan_range.append(scan_both[i])
        self.scan_range = scan_range

    def setVel(self, action):
        vel_cmd = Twist()
        # action(0〜4)によって並進速度を設定
        vel_cmd.linear.x = self.max_linear_vel \
            * [0.5, 1., 1., 1., 0.5][action]
        # action(0〜4)によって角速度を設定
        vel_cmd.angular.z = self.max_angular_vel \
            * [1., 0.5, 0., -0.5, -1.][action]
        return vel_cmd

    def pubInitVel(self):
        # dqnEnv_*.pyのdef setReward()からコールされてvel_cmdのnull値をパブリッシュ
        self.pub_cmd_vel.publish(Twist())
