#!/usr/bin/env python

import rospy
from geometry_msgs.msg import Twist
from sensor_msgs.msg import LaserScan


class baseDrive(object):
    def __init__(self, cmd_vel_name):
        self.max_linear_vel = 0.3
        self.max_angular_vel = 1.5
        self.scan_data2 = []
        self.scan_min = []
        self.last_scan_min = []
        self.corn_min = []
        self.keep_turn = 0
        self.pub_cmd_vel = rospy.Publisher(cmd_vel_name, Twist, queue_size=10)
        self.scan = rospy.Subscriber('scan', LaserScan, self.getScan)
        self.scan2 = rospy.Subscriber('scan2', LaserScan, self.getScan2)

    def getScan(self, scan_data):
        # self.corn_minを取得する
        scan_angle = [100, 160, 200, 260]
        # corn(後方角)のminを取得
        scan_corn_left = min(scan_data.ranges[scan_angle[0]:scan_angle[1]])
        scan_corn_right = min(scan_data.ranges[scan_angle[2]:scan_angle[3]])
        self.corn_min = [scan_corn_left, scan_corn_right]

    def getScan2(self, scan_data2):
        # 子クラスactDriveのdef getScan()でも使用するためにインスタンス変数にする
        self.scan_data2 = scan_data2
        # self.scan_minを取得する
        scan_angle = [10, 350, 20, 340, 50, 310]
        # front(正面)のminとmin_turn(回避旋回方向)を取得
        scan_front_left = min(scan_data2.ranges[0:scan_angle[0]])
        scan_front_right = min(scan_data2.ranges[scan_angle[1]:360])
        scan_front = min(scan_front_left, scan_front_right)
        front_min_turn = -1. if scan_front_left < scan_front_right else 1.
        # fwd(前方)のminとmin_turn(回避旋回方向)を取得
        scan_fwd_left = min(scan_data2.ranges[0:scan_angle[2]])
        scan_fwd_right = min(scan_data2.ranges[scan_angle[3]:360])
        scan_fwd = min(scan_fwd_left, scan_fwd_right)
        fwd_min_turn = -1. if scan_fwd_left < scan_fwd_right else 1.
        # side(側方)のminとmin_turn(回避旋回方向)を取得
        scan_side_left = min(scan_data2.ranges[0:scan_angle[4]])
        scan_side_right = min(scan_data2.ranges[scan_angle[5]:360])
        scan_side = min(scan_side_left, scan_side_right)
        side_min_turn = -1. if scan_side_left < scan_side_right else 1.
        self.scan_min = [scan_front, front_min_turn,
                         scan_fwd, fwd_min_turn,
                         scan_side, side_min_turn]

    def setVel(self, action):
        vel_cmd = Twist()
        check_front_dist = 0.7
        check_forward_dist = 0.5
        # 前方の障害物までの距離が所定値未満の場合は反対方向に急旋回進行
        if self.scan_min[2] < check_forward_dist:
            vel_cmd.linear.x = self.max_linear_vel * 0.5
            vel_cmd.angular.z = self.max_angular_vel * self.scan_min[3]
        # 正面の障害物までの距離が所定値未満の場合は反対方向に緩旋回進行
        elif self.scan_min[0] < check_front_dist:
            vel_cmd.linear.x = self.max_linear_vel
            vel_cmd.angular.z = self.max_angular_vel * self.scan_min[1] * 0.5
        # 障害物がなければ最大並進速度で直進
        else:
            vel_cmd.linear.x = self.max_linear_vel
            vel_cmd.angular.z = 0.
        return vel_cmd

    def pubVel(self, action=None):
        vel_cmd = Twist()
        check_side_dist = 0.31
        check_bc_dist = 0.17
        keep = ''   # keepは進路選択状況を見やすく表示する文字列
        # self.getScan2()がコールバックされてself.scan_minが書換えられるまで待つ
        while self.scan_min == self.last_scan_min:
            rospy.sleep(0.01)
        self.last_scan_min = self.scan_min
        # 側方の障害物までの距離が所定値以上の場合は進行方向を設定
        if self.scan_min[4] >= check_side_dist:
            self.keep_turn = 0
            if action is None:
                keep = 'ooo'
            else:
                keep = str(action) * 3
            vel_cmd = self.setVel(action)
        # 側方の障害物までの距離が所定値未満の場合はその場旋回
        else:
            # その場旋回中でない場合は障害物と反対方向にその場旋回方向を決定
            if self.keep_turn == 0:
                self.keep_turn = self.scan_min[5]
            # その場旋回で後方角が障害物に接近すると5秒待って旋回方向を反転
            elif self.keep_turn == 1 and self.corn_min[1] < check_bc_dist or \
                    self.keep_turn == -1 and self.corn_min[0] < check_bc_dist:
                rospy.sleep(5.)
                self.keep_turn *= -1
            # self.keep_turnに応じて旋回方向を決定
            keep = '+++' if self.keep_turn == 1 else '---'
            vel_cmd.linear.x = 0
            vel_cmd.angular.z = self.max_angular_vel * self.keep_turn
        self.pub_cmd_vel.publish(vel_cmd)
        return keep
